import React from 'react'
import { usePicker } from '../context.js'
import { formatInputValues } from '../utils/formatters.js'
import { controlBtnStyles } from './Controls.js'
import TrashIcon, {
  LinearIcon,
  RadialIcon,
  DegreesIcon,
  StopIcon,
} from './icon.js'

const GradientType = () => {
  const { gradientType, internalOnChange, value } = usePicker()
  const isLinear = gradientType === 'linear-gradient'
  const isRadial = gradientType === 'radial-gradient'

  const handleLinear = () => {
    const remaining = value.split(/,(.+)/)[1]
    internalOnChange(`linear-gradient(90deg, ${remaining}`)
  }

  const handleRadial = () => {
    const remaining = value.split(/,(.+)/)[1]
    internalOnChange(`radial-gradient(circle, ${remaining}`)
  }

  return (
    <div
      className="rbgcp-control-btn-wrapper"
      style={{
        height: '28px',
        // background: [theme]?.c3,
        borderRadius: '6px',
        padding: '2px',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'relative',
      }}
    >
      <div
        onClick={handleLinear}
        id="rbgcp-linear-btn"
        className={`rbgcp-control-btn rbgcp-linear-btn ${
          isLinear && 'rbgcp-control-btn-selected'
        }`}
        style={{
          paddingLeft: '8px',
          paddingRight: '8px',
          lineHeight: 1,
          borderRadius: '4px',
          fontWeight: 700,
          fontSize: '12px',
          height: '24px',
          transition: 'all 160ms ease',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          background: 'rgba(255,255,255,0)',
          boxShadow: '1px 1px 3px rgba(0,0,0,0)',
          // color: [theme]?.c1,
        }}
      >
        <LinearIcon color={isLinear ? '#568CF5' : ''} />
      </div>
      <div
        onClick={handleRadial}
        id="rbgcp-gradient-btn"
        className={`rbgcp-control-btn rbgcp-gradient-btn ${
          isRadial && 'rbgcp-control-btn-selected'
        }`}
        style={{
          paddingLeft: '8px',
          paddingRight: '8px',
          lineHeight: 1,
          borderRadius: '4px',
          fontWeight: 700,
          fontSize: '12px',
          height: '24px',
          transition: 'all 160ms ease',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          background: 'rgba(255,255,255,0)',
          boxShadow: '1px 1px 3px rgba(0,0,0,0)',
          // color: [theme]?.c1,
        }}
      >
        <RadialIcon color={isRadial ? '#568CF5' : ''} />
      </div>
    </div>
  )
}

const StopPicker = () => {
  const { currentLeft, handleGradient, currentColor } = usePicker()

  const handleMove = (newVal: string) => {
    handleGradient(currentColor, formatInputValues(parseInt(newVal), 0, 100))
  }

  return (
    <div
      className="rbgcp-control-btn-wrapper rbgcp-control-input-wrap rbgcp-stop-input-wrap"
      style={{
        paddingLeft: 8,
        height: '28px',
        // background: [theme]?.c3,
        borderRadius: '6px',
        padding: '2px',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'relative',
      }}
    >
      <StopIcon />
      <input
        id="rbgcp-stop-input"
        value={currentLeft}
        className="rbgcp-control-input rbgcp-stop-input"
        onChange={(e) => handleMove(e.target.value)}
        style={{
          height: '24px',
          borderRadius: '4px',
          border: 'none',
          outline: 'none',
          textAlign: 'center',
          width: '34px',
          fontWeight: 500,
          // color: [theme]?.c2,
          fontSize: '13px',
          background: 'transparent',
        }}
      />
    </div>
  )
}

const DegreePicker = () => {
  const { degrees, internalOnChange, value } = usePicker()

  const handleDegrees = (e: any) => {
    const newValue = formatInputValues(e.target.value, 0, 360)
    const remaining = value.split(/,(.+)/)[1]
    internalOnChange(`linear-gradient(${newValue || 0}deg, ${remaining}`)
  }

  return (
    <div
      className="rbgcp-control-btn-wrapper rbgcp-control-input-wrap rbgcp-degree-input-wrap"
      style={{
        paddingLeft: 8,
        height: '28px',
        // background: [theme]?.c3,
        borderRadius: '6px',
        padding: '2px',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'relative',
      }}
    >
      <DegreesIcon />
      <input
        id="rbgcp-degree-input"
        value={degrees}
        className="rbgcp-control-input rbgcp-degree-input"
        onChange={(e) => handleDegrees(e)}
        style={{
          height: '24px',
          borderRadius: '4px',
          border: 'none',
          outline: 'none',
          textAlign: 'center',
          width: '34px',
          fontWeight: 500,
          // color: [theme]?.c2,
          fontSize: '13px',
          background: 'transparent',
        }}
      />
      <div
        style={{
          position: 'absolute',
          right: degrees > 99 ? 0 : degrees < 10 ? 7 : 3,
          top: 1,
          fontWeight: 400,
          fontSize: 13,
        }}
        className="rbgcp-degree-icon"
      >
        °
      </div>
    </div>
  )
}

const DeconsteBtn = () => {
  const { deletePoint } = usePicker()

  return (
    <div
      onClick={deletePoint}
      style={{
        width: 28,

        position: 'relative',
        ...controlBtnStyles(false),
      }}
      id="rbgcp-point-deconste-btn"
      className="rbgcp-control-icon-btn rbgcp-point-deconste-btn"
    >
      <TrashIcon />
    </div>
  )
}

const GradientControls = ({
  hideGradientType,
  hideGradientAngle,
  hideGradientStop,
}: {
  hideGradientType?: boolean
  hideGradientAngle?: boolean
  hideGradientStop?: boolean
}) => {
  const { gradientType } = usePicker()
  return (
    <div
      style={{
        marginTop: 12,
        marginBottom: -4,
        paddingLeft: hideGradientType ? 4 : 0,
        display: 'flex',
        justifyContent: 'space-between',
        height: '28px',
        // background: [theme]?.c3,
        borderRadius: '6px',
        padding: '2px',
        alignItems: 'center',
        position: 'relative',
      }}
      id="rbgcp-gradient-controls-wrap"
      className="rbgcp-control-btn-wrapper"
    >
      {!hideGradientType && <GradientType />}
      <div style={{ width: 53 }}>
        {!hideGradientAngle && gradientType === 'linear-gradient' && (
          <DegreePicker />
        )}
      </div>
      {!hideGradientStop && <StopPicker />}
      <DeconsteBtn />
    </div>
  )
}

export default GradientControls
